#!/usr/bin/env python3
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import logging

from subunit import ByteStreamToStreamResult
from testtools import StreamToExtendedDecorator, TestResult


logging.basicConfig(level='INFO')
logger = logging.getLogger(__name__)


class FileSaveResult(TestResult):
    """A TestResult that saves arbitrary test details to Swift."""

    def __init__(self, data_store):
        super().__init__()
        self.data_store = data_store

    def addError(self, test, err=None, details=None):
        super().addError(test, err=err, details=details)
        self.save_file_attachments(test.id(), details)

    def addExpectedFailure(self, test, err=None, details=None):
        super().addExpectedFailure(test, err=err, details=details)
        self.save_file_attachments(test.id(), details)

    def addFailure(self, test, err=None, details=None):
        super().addFailure(test, err=err, details=details)
        self.save_file_attachments(test.id(), details)

    def addSkip(self, test, reason=None, details=None):
        super().addSkip(test, reason=reason, details=details)
        self.save_file_attachments(test.id(), details)

    def addSuccess(self, test, details=None):
        super().addSuccess(test, details=details)
        self.save_file_attachments(test.id(), details)

    def addUnexpectedSuccess(self, test, details=None):
        super().addUnexpectedSuccess(test, details=details)
        self.save_file_attachments(test.id(), details)

    def save_file_attachments(self, test_id, details):
        if details is not None:
            for key, detail in details.items():
                if detail.iter_bytes() != [b'']:
                    contents = b''.join(chunk for chunk in detail.iter_bytes())
                    self.data_store.put_file(
                        test_id, contents, detail.content_type)


def run_result(data_store, subunit_stream):
    suite = ByteStreamToStreamResult(subunit_stream)
    result = StreamToExtendedDecorator(FileSaveResult(data_store))
    result.startTestRun()
    try:
        suite.run(result)
    finally:
        result.stopTestRun()
        subunit_stream.close()
    return True
